@@ -7,13 +7,14 @@ CubeSet is the main API class for manipulating Crunch.io JSON cube responses.
 
 from __future__ import absolute_import, division, print_function, unicode_literals
 
+import copy
 import json
 
 import numpy as np
 
 from cr.cube.cubepart import CubePartition
 from cr.cube.dimension import AllDimensions
-from cr.cube.enum import DIMENSION_TYPE as DT
+from cr.cube.enums import CUBE_MEASURE, DIMENSION_TYPE as DT, NUMERIC_MEASURES
 from cr.cube.util import lazyproperty
 
 np.seterr(divide="ignore", invalid="ignore")
@@ -43,6 +44,11 @@ class CubeSet(object):
         self._population = population
         self._min_base = min_base
 
+    @lazyproperty
+    def available_measures(self):
+        """frozenset of available measures of the first cube in this set."""
+        return self._cubes[0].available_measures
+
     @lazyproperty
     def can_show_pairwise(self):
         """True if all 2D cubes in a multi-cube set can provide pairwise comparison."""
@@ -60,15 +66,10 @@ class CubeSet(object):
         """str description of first cube in this set."""
         return self._cubes[0].description
 
-    @lazyproperty
-    def has_means(self):
-        """True if cubes in this set include a means measure."""
-        return self._cubes[0].has_means
-
     @lazyproperty
     def has_weighted_counts(self):
         """True if cube-responses include a weighted-count measure."""
-        return self._cubes[0].is_weighted
+        return self._cubes[0].has_weighted_counts
 
     @lazyproperty
     def is_ca_as_0th(self):
@@ -131,10 +132,39 @@ class CubeSet(object):
         """
         return self._cubes[0].population_fraction
 
+    @lazyproperty
+    def n_responses(self):
+        """Total number of responses considered from first cube in this set."""
+        return self._cubes[0].n_responses
+
+    @lazyproperty
+    def valid_counts_summary(self):
+        """The valid count summary values from first cube in this set."""
+        return self._cubes[0].valid_counts_summary
+
     @lazyproperty
     def _cubes(self):
         """Sequence of Cube objects containing data for this analysis."""
-        return tuple(self._iter_cubes())
+
+        def iter_cubes():
+            """Generate a Cube object for each of cube_responses.
+
+            0D cube-responses and 1D second-and-later cubes are "inflated" to add their
+            missing row dimension.
+            """
+            for idx, cube_response in enumerate(self._cube_responses):
+                cube = Cube(
+                    cube_response,
+                    cube_idx=idx if self._is_multi_cube else None,
+                    transforms=self._transforms_dicts[idx],
+                    population=self._population,
+                    mask_size=self._min_base,
+                )
+                # --- numeric-measures cubes require inflation to restore their
+                # --- rows-dimension, others don't
+                yield cube.inflate() if self._is_numeric_measure else cube
+
+        return tuple(iter_cubes())
 
     @lazyproperty
     def _is_multi_cube(self):
@@ -142,10 +172,11 @@ class CubeSet(object):
         return len(self._cube_responses) > 1
 
     @lazyproperty
-    def _is_numeric_mean(self):
-        """True when CubeSet is special-case "numeric-mean" case requiring inflation.
+    def _is_numeric_measure(self):
+        """True when CubeSet is special-case "numeric-measure" case requiring inflation.
 
-        When a numeric variable appears as the rows-dimension in a multitable analysis,
+        When a numeric variable with `mean`, `sum` or `std_dev` summary statistic
+        expressed in its view, appears as the rows-dimension in a multitable analysis,
         its cube-result has been "reduced" to the mean-value of those numerics. This is
         in contrast to being "bucketized" into an arbitrary set of numeric-range
         categories like 0-5, 5-10, etc. In the process, as an artifact of the ZZ9 query
@@ -166,24 +197,6 @@ class CubeSet(object):
         # --- construction is low-overhead because all Cube properties are lazy.
         return Cube(self._cube_responses[0]).ndim == 0
 
-    def _iter_cubes(self):
-        """Generate a Cube object for each of cube_responses.
-
-        0D cube-responses and 1D second-and-later cubes are "inflated" to add their
-        missing row dimension.
-        """
-        for idx, cube_response in enumerate(self._cube_responses):
-            cube = Cube(
-                cube_response,
-                cube_idx=idx if self._is_multi_cube else None,
-                transforms=self._transforms_dicts[idx],
-                population=self._population,
-                mask_size=self._min_base,
-            )
-            # --- all numeric-mean cubes require inflation to restore their
-            # --- rows-dimension, others don't
-            yield cube.inflate() if self._is_numeric_mean else cube
-
 
 class Cube(object):
     """Provides access to individual slices on a cube-result.
@@ -220,13 +233,30 @@ class Cube(object):
         except Exception:
             return super(Cube, self).__repr__()
 
+    @lazyproperty
+    def available_measures(self):
+        """frozenset of available CUBE_MEASURE members in the cube response."""
+        cube_measures = self._cube_response.get("result", {}).get("measures", {}).keys()
+        return frozenset(CUBE_MEASURE(m) for m in cube_measures)
+
     @lazyproperty
     def counts(self):
         return self.counts_with_missings[self._valid_idxs]
 
     @lazyproperty
     def counts_with_missings(self):
-        return self._measure(self.is_weighted).raw_cube_array
+        """ndarray of weighted, unweighted or valid counts including missing values.
+
+        The difference from .counts is that this property includes value for missing
+        categories.
+        """
+        return (
+            self._measures.valid_counts.raw_cube_array
+            if self._measures.valid_counts is not None
+            else self._measures.weighted_counts.raw_cube_array
+            if self.has_weighted_counts
+            else self._measures.unweighted_counts.raw_cube_array
+        )
 
     @lazyproperty
     def cube_index(self):
@@ -258,11 +288,6 @@ class Cube(object):
         """
         return self._all_dimensions.apparent_dimensions
 
-    @lazyproperty
-    def has_means(self):
-        """True if cube includes a means measure."""
-        return self._measures.means is not None
-
     def inflate(self):
         """Return new Cube object with rows-dimension added.
 
@@ -271,9 +296,17 @@ class Cube(object):
         """
         cube_dict = self._cube_dict
         dimensions = cube_dict["result"]["dimensions"]
+        default_name = "-".join([m.value for m in self._available_numeric_measures])
+        # --- The default value in case of numeric variable is the combination of all
+        # --- the measures expressed in the cube response.
+        alias = self._numeric_measure_references.get("alias", default_name)
+        name = self._numeric_measure_references.get("name", default_name).title()
         rows_dimension = {
-            "references": {"alias": "mean", "name": "mean"},
-            "type": {"categories": [{"id": 1, "name": "Mean"}], "class": "categorical"},
+            "references": {"alias": alias, "name": name},
+            "type": {
+                "categories": [{"id": 1, "name": name}],
+                "class": "categorical",
+            },
         }
         dimensions.insert(0, rows_dimension)
         return Cube(
@@ -285,32 +318,16 @@ class Cube(object):
         )
 
     @lazyproperty
-    def is_mr_aug(self):
-        """True if cube is MR_AUG (augmented).
-
-        An augmented-MR cube is required to compute pairwise-t-tests when an MR
-        dimension is involved because that introduces *overlap* which otherwise inflates
-        the case count.
-
-        An MR_AUG cube is used to calculate statistical significance of complete-cases,
-        and is created by artificially *augmenting* the original cube's dimensions, such
-        as MR, or CAT_X_MR, by repeating the MR dimension to make the cube MR_X_MR or
-        CAT_X_MR_X_MR for these respective examples. The repeating of the MR dimension
-        allows the overlaps to be computed.
-        """
-        return (
-            # --- there are at least three dimensions ---
-            self.ndim >= 3
-            # --- the last two are both MR ---
-            and all(dim_type == DT.MR for dim_type in self.dimension_types[-2:])
-            # --- and they both have the same alias ---
-            and len(set(dim.alias for dim in self.dimensions[-2:])) == 1
-        )
+    def has_weighted_counts(self):
+        """True if cube response has weighted count data."""
+        return self.weighted_counts is not None
 
     @lazyproperty
-    def is_weighted(self):
-        """True if cube response contains weighted data."""
-        return self._measures.is_weighted
+    def means(self):
+        """Optional float64 ndarray of the cube_means if the measure exists."""
+        if self._measures.means is None:
+            return None
+        return self._measures.means.raw_cube_array[self._valid_idxs].astype(np.float64)
 
     @lazyproperty
     def missing(self):
@@ -334,6 +351,11 @@ class Cube(object):
         """int count of dimensions for this cube."""
         return len(self.dimensions)
 
+    @lazyproperty
+    def n_responses(self):
+        """Total (int) number of responses considered."""
+        return self._cube_response["result"].get("n", 0)
+
     @lazyproperty
     def partitions(self):
         """Sequence of _Slice, _Strand, or _Nub objects from this cube-result."""
@@ -360,6 +382,13 @@ class Cube(object):
         """
         return self._measures.population_fraction
 
+    @lazyproperty
+    def sums(self):
+        """Optional float64 ndarray of the cube_sum if the measure exists."""
+        if self._measures.sums is None:
+            return None
+        return self._measures.sums.raw_cube_array[self._valid_idxs].astype(np.float64)
+
     @lazyproperty
     def title(self):
         """str alternate-name given to cube-result.
@@ -375,11 +404,49 @@ class Cube(object):
         """ndarray of unweighted counts, valid elements only.
 
         Unweighted counts are drawn from the `result.counts` field of the cube result.
-        These counts are always present, even when the measure is `mean` and there are
+        These counts are always present, even when the measure is numeric and there are
         no count measures. These counts are always unweighted, regardless of whether the
         cube is "weighted".
+
+        In case of presence of valid counts in the cube response the counts are replaced
+        with the valid counts measure.
         """
-        return self._measures.unweighted_counts.raw_cube_array[self._valid_idxs]
+        unweighted_counts = (
+            self._measures.valid_counts
+            if self._measures.valid_counts is not None
+            else self._measures.unweighted_counts
+        )
+        return unweighted_counts.raw_cube_array[self._valid_idxs]
+
+    @lazyproperty
+    def valid_counts_summary(self):
+        """Optional ndarray of summary valid counts"""
+        if not self._measures.valid_counts:
+            return None
+        # --- In case of ndim >= 2 the sum should be done on the second axes to get
+        # --- the correct sequence of valid count (e.g. CA_SUBVAR).
+        axis = 1 if len(self._all_dimensions) >= 2 else 0
+        return np.sum(
+            self._measures.valid_counts.raw_cube_array[self._valid_idxs], axis=axis
+        )
+
+    @lazyproperty
+    def weighted_counts(self):
+        """ndarray of weighted counts, valid elements only.
+
+        In case of presence of valid counts in the cube response the weighted counts
+        are replaced with the valid counts measure.
+        """
+        weighted_counts = (
+            self._measures.valid_counts
+            if self._measures.valid_counts is not None
+            else self._measures.weighted_counts
+        )
+        return (
+            weighted_counts.raw_cube_array[self._valid_idxs]
+            if self._measures.weighted_counts is not None
+            else None
+        )
 
     @lazyproperty
     def _all_dimensions(self):
@@ -393,6 +460,15 @@ class Cube(object):
         """
         return AllDimensions(dimension_dicts=self._cube_dict["result"]["dimensions"])
 
+    @lazyproperty
+    def _available_numeric_measures(self):
+        """tuple of available numeric measures expressed in the cube_response.
+
+        Basically the numeric measures are the intersection between all the measures
+        within the cube response and the defined NUMERIC_MEASURES.
+        """
+        return tuple(self.available_measures.intersection(NUMERIC_MEASURES))
+
     @lazyproperty
     def _ca_as_0th(self):
         """True if slicing is to be performed in so-called "CA-as-0th" mode.
@@ -410,16 +486,27 @@ class Cube(object):
     @lazyproperty
     def _cube_dict(self):
         """dict containing raw cube response, parsed from JSON payload."""
+        cube_dict = copy.deepcopy(self._cube_response)
+        if self._numeric_measure_subvariables:
+            dimensions = cube_dict.get("result", {}).get("dimensions", [])
+            # ---dim inflation---
+            # ---In case of numeric arrays, we need to inflate the row dimension
+            # ---according to the mean subvariables. For each subvar the row dimension
+            # ---will have a new element related to the subvar metadata.
+            dimensions.insert(0, self._numeric_array_dimension)
+        return cube_dict
+
+    @lazyproperty
+    def _cube_response(self):
+        """dict representing the parsed cube response arguments."""
         try:
-            cube_response = self._cube_response_arg
+            response = self._cube_response_arg
             # ---parse JSON to a dict when constructed with JSON---
-            cube_dict = (
-                cube_response
-                if isinstance(cube_response, dict)
-                else json.loads(cube_response)
+            cube_response = (
+                response if isinstance(response, dict) else json.loads(response)
             )
             # ---cube is 'value' item in a shoji response---
-            return cube_dict.get("value", cube_dict)
+            return cube_response.get("value", cube_response)
         except TypeError:
             raise TypeError(
                 "Unsupported type <%s> provided. Cube response must be JSON "
@@ -431,33 +518,71 @@ class Cube(object):
         """bool determines if it is a single column filter cube."""
         return self._cube_dict["result"].get("is_single_col_cube", False)
 
-    def _measure(self, weighted):
-        """_BaseMeasure subclass representing primary measure for this cube.
-
-        If the cube response includes a means measure, the return value is
-        means. Otherwise it is counts, with the choice between weighted or
-        unweighted determined by *weighted*.
-
-        Note that weighted counts are provided on an "as-available" basis.
-        When *weighted* is True and the cube response is not weighted,
-        unweighted counts are returned.
-        """
-        return (
-            self._measures.means
-            if self._measures.means is not None
-            else self._measures.weighted_counts
-            if weighted
-            else self._measures.unweighted_counts
-        )
-
     @lazyproperty
     def _measures(self):
         """_Measures object for this cube.
 
-        Provides access to unweighted counts, and weighted counts and/or means
+        Provides access to count based measures and numeric measures (e.g. mean, sum)
         when available.
         """
-        return _Measures(self._cube_dict, self._all_dimensions)
+        return _Measures(self._cube_dict, self._all_dimensions, self._cube_idx_arg)
+
+    @lazyproperty
+    def _numeric_measure_references(self):
+        """Dict of numeric measure references, typically for numeric measures."""
+        if not self._available_numeric_measures:
+            return {}
+        cube_response = self._cube_response
+        cube_measures = cube_response.get("result", {}).get("measures", {})
+        metadata = cube_measures.get(self._available_numeric_measures[0].value, {}).get(
+            "metadata", {}
+        )
+        return metadata.get("references", {})
+
+    @lazyproperty
+    def _numeric_measure_subvariables(self):
+        """List of mean subvariables, typically for numeric arrays."""
+        if not self._available_numeric_measures:
+            return []
+        cube_response = self._cube_response
+        cube_measures = cube_response.get("result", {}).get("measures", {})
+        metadata = cube_measures.get(self._available_numeric_measures[0].value, {}).get(
+            "metadata", {}
+        )
+        return metadata.get("type", {}).get("subvariables", [])
+
+    @lazyproperty
+    def _numeric_array_dimension(self):
+        """Rows dimension object according to the numeric-measure subvariables."""
+        if not self._numeric_measure_subvariables:
+            return None
+        subrefs = self._numeric_measure_references.get("subreferences", [])
+        rows_dimension = {
+            "references": {
+                "alias": self._numeric_measure_references.get("alias"),
+                "name": self._numeric_measure_references.get("name"),
+            },
+            "type": {"elements": [], "class": "enum", "subtype": {"class": "num_arr"}},
+        }
+        # ---In case of numeric arrays the row dimension should contains additional
+        # ---information related to the subreferences for each subvariable of the
+        # ---array.
+        for i, _ in enumerate(self._numeric_measure_subvariables):
+            # ---The row dimensions elements must be expanded with the alias and the
+            # ---name of the numeric array mean measure subreferences.
+            rows_dimension["type"].get("elements", []).append(
+                {
+                    "id": i,
+                    "value": {
+                        "references": {
+                            "alias": subrefs[i].get("alias") if subrefs else None,
+                            "name": subrefs[i].get("name") if subrefs else None,
+                        },
+                        "id": self._numeric_measure_subvariables[i],
+                    },
+                },
+            )
+        return rows_dimension
 
     @lazyproperty
     def _slice_idxs(self):
@@ -472,56 +597,39 @@ class Cube(object):
 
     @lazyproperty
     def _valid_idxs(self):
-        return np.ix_(
+        """Tuple of int64 ndarrays of the valid elements idx for each dimension."""
+        valid_idxs = np.ix_(
             *tuple(d.valid_elements.element_idxs for d in self._all_dimensions)
         )
+        # The dimension dimension order can change in case of numeric array variable on
+        # the row, and so valid indices needs to be returned in an ordered way.
+        return tuple(valid_idxs[i] for i in self._all_dimensions.dimension_order)
 
 
 class _Measures(object):
     """Provides access to measures contained in cube response."""
 
-    def __init__(self, cube_dict, all_dimensions):
+    def __init__(self, cube_dict, all_dimensions, cube_idx_arg=None):
         self._cube_dict = cube_dict
         self._all_dimensions = all_dimensions
-
-    @lazyproperty
-    def is_weighted(self):
-        """True if weights have been applied to the measure(s) for this cube.
-
-        Unweighted counts are available for all cubes. Weighting applies to
-        any other measures provided by the cube.
-        """
-        cube_dict = self._cube_dict
-        if cube_dict.get("query", {}).get("weight") is not None:
-            return True
-        if cube_dict.get("weight_var") is not None:
-            return True
-        if cube_dict.get("weight_url") is not None:
-            return True
-        unweighted_counts = cube_dict["result"]["counts"]
-        count_data = cube_dict["result"]["measures"].get("count", {}).get("data")
-        if unweighted_counts != count_data:
-            return True
-        return False
+        self._cube_idx_arg = cube_idx_arg
 
     @lazyproperty
     def means(self):
-        """_MeanMeasure object providing access to means values.
+        """Optional _MeanMeasure object providing access to means values.
 
-        None when the cube response does not contain a mean measure.
+        Will be None if no means are available on the counts.
         """
-        mean_measure_dict = (
-            self._cube_dict.get("result", {}).get("measures", {}).get("mean")
-        )
-        if mean_measure_dict is None:
-            return None
-        return _MeanMeasure(self._cube_dict, self._all_dimensions)
+        mean = _MeanMeasure(self._cube_dict, self._all_dimensions, self._cube_idx_arg)
+        return None if mean.raw_cube_array is None else mean
 
     @lazyproperty
     def missing_count(self):
         """numeric representing count of missing rows in cube response."""
-        if self.means:
+        if self.means is not None:
             return self.means.missing_count
+        if self.sums is not None:
+            return self.sums.missing_count
         return self._cube_dict["result"].get("missing", 0)
 
     @lazyproperty
@@ -564,6 +672,15 @@ class _Measures(object):
         except Exception:
             return 1.0
 
+    @lazyproperty
+    def sums(self):
+        """_SumMeasure object providing access to cube sum values.
+
+        None when the cube response does not contain a sum measure.
+        """
+        sums = _SumMeasure(self._cube_dict, self._all_dimensions, self._cube_idx_arg)
+        return None if sums.raw_cube_array is None else sums
+
     @lazyproperty
     def unweighted_counts(self):
         """_UnweightedCountMeasure object for this cube.
@@ -571,46 +688,60 @@ class _Measures(object):
         This object provides access to unweighted counts for this cube,
         whether or not the cube contains weighted counts.
         """
-        return _UnweightedCountMeasure(self._cube_dict, self._all_dimensions)
+        return _UnweightedCountMeasure(
+            self._cube_dict, self._all_dimensions, self._cube_idx_arg
+        )
+
+    @lazyproperty
+    def valid_counts(self):
+        """_ValidCountsMeasure object for this cube.
+
+        Can be None when cube doesn't have valid counts.
+        """
+        valid_counts = _ValidCountsMeasure(
+            self._cube_dict, self._all_dimensions, self._cube_idx_arg
+        )
+        return valid_counts if valid_counts.raw_cube_array is not None else None
 
     @lazyproperty
     def weighted_counts(self):
-        """_WeightedCountMeasure object for this cube.
+        """Optional _WeightedCountMeasure object for this cube.
 
-        This object provides access to weighted counts for this cube, if
-        available. If the cube response is not weighted, the
-        _UnweightedCountMeasure object for this cube is returned.
+        Can be None when the cube is unweighted.
         """
-        return (
-            _WeightedCountMeasure(self._cube_dict, self._all_dimensions)
-            if self.is_weighted
-            else _UnweightedCountMeasure(self._cube_dict, self._all_dimensions)
+        weighted_counts = _WeightedCountMeasure(
+            self._cube_dict, self._all_dimensions, self._cube_idx_arg
         )
+        return weighted_counts if weighted_counts.raw_cube_array is not None else None
 
 
 class _BaseMeasure(object):
     """Base class for measure objects."""
 
-    def __init__(self, cube_dict, all_dimensions):
+    def __init__(self, cube_dict, all_dimensions, cube_idx_arg=None):
         self._cube_dict = cube_dict
         self._all_dimensions = all_dimensions
+        self._cube_idx_arg = cube_idx_arg
 
     @lazyproperty
     def raw_cube_array(self):
-        """Return read-only ndarray of measure values from cube-response.
+        """Optional read-only ndarray of measure values from cube-response.
 
         The shape of the ndarray mirrors the shape of the (raw) cube
         response. Specifically, it includes values for missing elements, any
-        MR_CAT dimensions, and any prunable rows and columns.
+        MR_CAT dimensions, and any prunable rows and columns. Returns None
+        if the measure is not available in cube.
         """
-        array = np.array(self._flat_values).reshape(self._all_dimensions.shape)
+        if self._flat_values is None:
+            return None
+        raw_cube_array = self._flat_values.reshape(self._all_dimensions.shape)
         # ---must be read-only to avoid hard-to-find bugs---
-        array.flags.writeable = False
-        return array
+        raw_cube_array.flags.writeable = False
+        return raw_cube_array
 
     @lazyproperty
     def _flat_values(self):  # pragma: no cover
-        """Return tuple of mean values as found in cube response.
+        """Return ndarray of np.float64 values as found in cube response.
 
         This property must be implemented by each subclass.
         """
@@ -622,21 +753,64 @@ class _MeanMeasure(_BaseMeasure):
 
     @lazyproperty
     def missing_count(self):
-        """numeric representing count of missing rows reflected in response."""
+        """Numeric value representing count of missing rows in response."""
         return self._cube_dict["result"]["measures"]["mean"].get("n_missing", 0)
 
     @lazyproperty
     def _flat_values(self):
-        """Return tuple of mean values as found in cube response.
+        """Optional 1D np.ndarray of np.float64 mean values as found in cube response.
 
         Mean data may include missing items represented by a dict like
         {'?': -1} in the cube response. These are replaced by np.nan in the
         returned value.
         """
-        return tuple(
-            np.nan if type(x) is dict else x
-            for x in self._cube_dict["result"]["measures"]["mean"]["data"]
+        measure_payload = self._cube_dict["result"].get("measures", {}).get("mean")
+        if measure_payload is None:
+            return None
+        return np.array(
+            tuple(np.nan if type(x) is dict else x for x in measure_payload["data"]),
+            dtype=np.float64,
+        ).flatten()
+
+
+class _SumMeasure(_BaseMeasure):
+    """Statistical sum values from a cube-response."""
+
+    @lazyproperty
+    def missing_count(self):
+        """numeric representing count of missing rows reflected in response."""
+        return self._cube_dict["result"]["measures"]["sum"].get("n_missing", 0)
+
+    @lazyproperty
+    def _flat_values(self):
+        """Optional 1D float64 ndarray of sum values as found in cube response.
+
+        Sum data may include missing items represented by a dict like
+        {'?': -1} in the cube response. These are replaced by np.nan in the
+        returned value.
+        """
+        measure_payload = self._cube_dict["result"].get("measures", {}).get("sum")
+        if measure_payload is None:
+            return None
+
+        return np.array(
+            tuple(np.nan if type(x) is dict else x for x in measure_payload["data"]),
+            dtype=np.float64,
+        ).flatten()
+
+
+class _ValidCountsMeasure(_BaseMeasure):
+    """Valid counts for cube."""
+
+    @lazyproperty
+    def _flat_values(self):
+        """Optional 1D np.ndarray of np.float64 valid counts."""
+        valid_counts = (
+            self._cube_dict["result"]["measures"]
+            .get("valid_count_unweighted", {})
+            .get("data", [])
         )
+        return np.array(valid_counts, dtype=np.float64) if valid_counts else None
 
 
 class _UnweightedCountMeasure(_BaseMeasure):
@@ -644,8 +818,11 @@ class _UnweightedCountMeasure(_BaseMeasure):
 
     @lazyproperty
     def _flat_values(self):
-        """tuple of int counts before weighting."""
-        return tuple(self._cube_dict["result"]["counts"])
+        """1D np.ndarray of np.float64 counts before weighting.
+
+        Use np.float64s to avoid int overflow bugs and so we can use nan.
+        """
+        return np.array(self._cube_dict["result"]["counts"], dtype=np.float64)
 
 
 class _WeightedCountMeasure(_BaseMeasure):
@@ -653,5 +830,12 @@ class _WeightedCountMeasure(_BaseMeasure):
 
     @lazyproperty
     def _flat_values(self):
-        """tuple of numeric counts after weighting."""
-        return tuple(self._cube_dict["result"]["measures"]["count"]["data"])
+        """Optional 1D np.ndarray of np.float64 numeric counts after weighting."""
+        unweighted_counts = self._cube_dict["result"]["counts"]
+        weighted_counts = (
+            self._cube_dict["result"]["measures"].get("count", {}).get("data")
+        )
+        if unweighted_counts == weighted_counts or weighted_counts is None:
+            return None
+
+        return np.array(weighted_counts, dtype=np.float64)
