@@ -5,13 +5,13 @@
 import sys
 
 if sys.version_info >= (3, 3):
-    from collections.abc import Sequence
+    from collections.abc import Sequence  # pragma: no cover
 else:
-    from collections import Sequence
+    from collections import Sequence  # pragma: no cover
 
 import numpy as np
 
-from cr.cube.enum import DIMENSION_TYPE as DT
+from cr.cube.enums import COLLATION_METHOD as CM, DIMENSION_TYPE as DT
 from cr.cube.util import lazyproperty
 
 
@@ -57,6 +57,30 @@ class AllDimensions(_BaseDimensions):
         """
         return _ApparentDimensions(all_dimensions=self._dimensions)
 
+    @lazyproperty
+    def dimension_order(self):
+        """Tuple of int representing the dimension order.
+
+        The dimension order depends on the presence of numeric array in the dimensions
+        and the number of the cube dimensions. In case of 3 dimensions e.g.
+        NUM_ARR_X_MR_SUBVAR_X_MR_CAT the order should be (1,2,0) that is basically
+        swapping the MR (2 dimensions) with the NUM_ARRAY dimension. In case of 2
+        dimensions the dimension order correspond simpy to the reverse of the original
+        dimension order.
+        """
+        # NOTE: this is a temporary hack that goes away when we introduce the dim_order
+        # concept. We should receive the actual order directly in the cube_response.
+        # So, all this logic will be deleted.
+        dimension_types = tuple(d.dimension_type for d in self._dimensions)
+        dim_order = tuple(range(len(self._dimensions)))
+        if len(self._dimensions) >= 2 and DT.NUM_ARRAY in dimension_types:
+            return (
+                dim_order[-2:] + (dim_order[0],)
+                if len(self._dimensions) == 3
+                else dim_order[::-1]
+            )
+        return dim_order
+
     @lazyproperty
     def shape(self):
         """Tuple of int element count for each dimension.
@@ -65,7 +89,8 @@ class AllDimensions(_BaseDimensions):
         cube response values (raw meaning including missing and prunable
         elements and any MR_CAT dimensions).
         """
-        return tuple(d.shape for d in self._dimensions)
+        dimensions = [self._dimensions[i] for i in self.dimension_order]
+        return tuple(d.shape for d in dimensions)
 
     @lazyproperty
     def _dimensions(self):
@@ -156,6 +181,8 @@ class _RawDimension(object):
             return DT.BINNED_NUMERIC
         if base_type == "enum.text":
             return DT.TEXT
+        if base_type == "enum.num_arr":
+            return DT.NUM_ARRAY
         raise NotImplementedError("unrecognized dimension type %s" % base_type)
 
     @lazyproperty
@@ -261,16 +288,12 @@ class Dimension(object):
     def __init__(self, dimension_dict, dimension_type, dimension_transforms=None):
         self._dimension_dict = dimension_dict
         self._dimension_type = dimension_type
-        self._dimension_transforms_arg = dimension_transforms
+        self._dimension_transforms_dict = dimension_transforms or {}
 
     @lazyproperty
     def alias(self):
-        """Return the alias for the dimension if it exists, None otherwise
-
-        This property is needed to identify one of the mandatory condition
-        for a MRxItself cube.
-        """
-        return self._dimension_dict["references"].get("alias", None)
+        """Return the alias for the dimension if it exists, None otherwise."""
+        return self._dimension_dict["references"].get("alias")
 
     @lazyproperty
     def all_elements(self):
@@ -293,6 +316,14 @@ class Dimension(object):
             self._dimension_dict, self._dimension_type, dimension_transforms
         )
 
+    @lazyproperty
+    def collation_method(self):
+        """Member of COLLATION_METHOD specifying ordering of dimension elements."""
+        method_keyword = self.order_dict.get("type")
+        if method_keyword is None:
+            return CM.PAYLOAD_ORDER
+        return CM(method_keyword)
+
     @lazyproperty
     def description(self):
         """str description of this dimension."""
@@ -338,6 +369,29 @@ class Dimension(object):
         """
         return self.valid_elements.display_order
 
+    @lazyproperty
+    def element_ids(self):
+        """tuple of int element-id for each valid element in this dimension.
+
+        Element-ids appear in the order defined in the cube-result.
+        """
+        return tuple(e.element_id for e in self.valid_elements)
+
+    @lazyproperty
+    def element_subvar_ids(self):
+        return tuple(e.element_subvar_id for e in self.valid_elements)
+
+    @lazyproperty
+    def hidden_idxs(self):
+        """tuple of int element-idx for each hidden valid element in this dimension.
+
+        An element is hidden when a "hide" transform is applied to it in its transforms
+        dict.
+        """
+        return tuple(
+            idx for idx, element in enumerate(self.valid_elements) if element.is_hidden
+        )
+
     @lazyproperty
     def name(self):
         """str name of this dimension, the empty string ("") if not specified."""
@@ -373,29 +427,32 @@ class Dimension(object):
         """
         return tuple(element.numeric_value for element in self.valid_elements)
 
+    @lazyproperty
+    def order_dict(self):
+        """dict dimension.transforms.order field parsed from JSON payload.
+
+        Value is `{}` if no "order": field is present.
+        """
+        return self._dimension_transforms_dict.get("order", {})
+
     @lazyproperty
     def prune(self):
         """True if empty elements should be automatically hidden on this dimension."""
-        prune = self._dimension_transforms_dict.get("prune")
-        if prune is True:
-            return True
-        return False
+        return self._dimension_transforms_dict.get("prune") is True
+
+    @lazyproperty
+    def selected_categories(self):
+        """List of selected categories specified for this dimension."""
+        selected_categories = self._dimension_dict["references"].get(
+            "selected_categories"
+        )
+        return tuple(selected_categories) if selected_categories else ()
 
     @lazyproperty
     def shape(self):
+        """int count of *all* elements in this dimension, both valid and missing."""
         return len(self.all_elements)
 
-    @lazyproperty
-    def sort(self):
-        """A _BaseSort-subclass object or None, describing the applied sort method.
-
-        This value is None if no sort transform was specified for this dimension.
-        Currently that is its only possible value. The returned sort object describes
-        the sort method which can include sorting on the value of an opposing element or
-        on the margin and specify ascending or descending order.
-        """
-        return None  # pragma: no cover
-
     @lazyproperty
     def subtotals(self):
         """_Subtotals sequence object for this dimension.
@@ -403,18 +460,19 @@ class Dimension(object):
         Each item in the sequence is a _Subtotal object specifying a subtotal, including
         its addends and anchor.
         """
-        # ---insertions in dimension-transforms override those on dimension itself---
-        insertion_dicts = self._dimension_transforms_dict.get("insertions")
-        if insertion_dicts is not None:
-            return _Subtotals(insertion_dicts, self.valid_elements, self.prune)
+        # --- elements of an aggregate/array dimension cannot meaningfully be summed, so
+        # --- an array dimension cannot have subtotals
+        if self.dimension_type in (DT.MR, DT.CA_SUBVAR):
+            insertion_dicts = []
+        # --- insertions in dimension-transforms override those on dimension itself ---
+        elif "insertions" in self._dimension_transforms_dict:
+            insertion_dicts = self._dimension_transforms_dict["insertions"]
+        # --- otherwise insertions defined on dimension/variable apply ---
+        else:
+            view = self._dimension_dict.get("references", {}).get("view") or {}
+            insertion_dicts = view.get("transform", {}).get("insertions", [])
 
-        # ---otherwise, insertions defined as default transforms apply---
-        view = self._dimension_dict.get("references", {}).get("view", {})
-        # ---view can be both None and {}, thus the edge case.---
-        insertion_dicts = (
-            [] if view is None else view.get("transform", {}).get("insertions", [])
-        )
-        return _Subtotals(insertion_dicts, self.valid_elements, self.prune)
+        return _Subtotals(insertion_dicts, self.valid_elements)
 
     @lazyproperty
     def valid_elements(self):
@@ -426,19 +484,6 @@ class Dimension(object):
         """
         return self.all_elements.valid_elements
 
-    @lazyproperty
-    def _dimension_transforms_dict(self):
-        """dict complying with dimension-transforms schema for this dimension.
-
-        This value derives from the `dimension_transforms` argument passed on
-        construction. When that argument is not specified, this value is an empty dict.
-        """
-        return (
-            self._dimension_transforms_arg
-            if self._dimension_transforms_arg is not None
-            else {}
-        )
-
 
 class _BaseElements(Sequence):
     """Base class for element sequence containers."""
@@ -527,7 +572,7 @@ class _AllElements(_BaseElements):
             _Element(
                 element_dict,
                 idx,
-                _ElementTransforms(element_transforms_dict, self._prune),
+                _ElementTransforms(element_transforms_dict),
             )
             for (
                 idx,
@@ -555,11 +600,6 @@ class _AllElements(_BaseElements):
             )
             yield idx, element_dict, element_transforms_dict
 
-    @lazyproperty
-    def _prune(self):
-        """True if empty elements in this dimension should be automatically hidden."""
-        return True if self._dimension_transforms_dict.get("prune") is True else False
-
 
 class _ValidElements(_BaseElements):
     """Sequence of non-missing element objects for a dimension.
@@ -653,6 +693,14 @@ class _Element(object):
         """int identifier for this category or subvariable."""
         return self._element_dict["id"]
 
+    @lazyproperty
+    def element_subvar_id(self):
+        """str identifier for the subvar ID (from ZZ9), or None if not a subvar."""
+        value = self._element_dict.get("value")
+        if value is None or not isinstance(value, dict):
+            return None
+        return value.get("id")
+
     @lazyproperty
     def fill(self):
         """str RGB color like "#af032d" or None if not specified.
@@ -740,18 +788,12 @@ class _Element(object):
         numeric_value = self._element_dict.get("numeric_value")
         return np.nan if numeric_value is None else numeric_value
 
-    @lazyproperty
-    def prune(self):
-        """True if this element should be hidden when empty, False otherwise."""
-        return self._element_transforms.prune
-
 
 class _ElementTransforms(object):
     """A value object providing convenient access to transforms for a single element."""
 
-    def __init__(self, element_transforms_dict, prune):
+    def __init__(self, element_transforms_dict):
         self._element_transforms_dict = element_transforms_dict
-        self._prune = prune
 
     @lazyproperty
     def fill(self):
@@ -794,11 +836,6 @@ class _ElementTransforms(object):
         name = self._element_transforms_dict["name"]
         return str(name) if name else ""
 
-    @lazyproperty
-    def prune(self):
-        """True if this element should be hidden when empty."""
-        return self._prune
-
 
 class _Subtotals(Sequence):
     """Sequence of _Subtotal objects for a dimension.
@@ -809,10 +846,9 @@ class _Subtotals(Sequence):
     A subtotal can only involve valid (i.e. non-missing) elements.
     """
 
-    def __init__(self, insertion_dicts, valid_elements, prune):
+    def __init__(self, insertion_dicts, valid_elements):
         self._insertion_dicts = insertion_dicts
         self._valid_elements = valid_elements
-        self._prune = prune
 
     def __getitem__(self, idx_or_slice):
         """Implements indexed access."""
@@ -826,40 +862,6 @@ class _Subtotals(Sequence):
         """Implements len(subtotals)."""
         return len(self._subtotals)
 
-    @lazyproperty
-    def anchor_idxs(self):
-        """List of int indicating the actual position of the subtotals."""
-        # TODO: this appears wrong, using an anchor (element_id) as an index (offset of
-        # anchor element within elements). This may appear to work if category-ids are
-        # assigned consecutively starting at 1, but will not work in the general case.
-        # I'll bet it also doesn't work when more than one subtotal is anchored to the
-        # same element-id.
-
-        def occurrences(s, lst):
-            return (i for i, e in enumerate(lst) if e == s)
-
-        anchors = self._anchors
-        top_anchors = list(occurrences("top", anchors))
-        int_anchors = [a + len(top_anchors) for a in anchors if isinstance(a, int)]
-        bottom_anchors = [
-            a + len(self._valid_elements) + len(top_anchors + int_anchors)
-            for a, _ in enumerate(list(occurrences("bottom", anchors)))
-        ]
-        final_anchors = top_anchors + int_anchors + bottom_anchors
-        return final_anchors
-
-    def iter_for_anchor(self, anchor):
-        """Generate each subtotal having matching *anchor*."""
-        return (subtotal for subtotal in self._subtotals if subtotal.anchor == anchor)
-
-    @lazyproperty
-    def _anchors(self):
-        """List of int or str indicating element under which to insert this subtotal."""
-        return list(
-            _Subtotal(subtotal_dict, self._valid_elements, self._prune).anchor
-            for subtotal_dict in self._iter_valid_subtotal_dicts()
-        )
-
     @lazyproperty
     def _element_ids(self):
         """frozenset of int id of each non-missing cat or subvar in dim."""
@@ -896,18 +898,23 @@ class _Subtotals(Sequence):
     def _subtotals(self):
         """Composed tuple storing actual sequence of _Subtotal objects."""
         return tuple(
-            _Subtotal(subtotal_dict, self._valid_elements, self._prune)
-            for subtotal_dict in self._iter_valid_subtotal_dicts()
+            _Subtotal(subtotal_dict, self._valid_elements, idx + 1)
+            for idx, subtotal_dict in enumerate(self._iter_valid_subtotal_dicts())
         )
 
 
 class _Subtotal(object):
-    """A subtotal insertion on a cube dimension."""
+    """A subtotal insertion on a cube dimension.
 
-    def __init__(self, subtotal_dict, valid_elements, prune):
+    `fallback_insertion_id` is a fallback unique identifier for this insertion, until
+    real insertion-ids can be added. Its value is just the index+1 of this subtotal
+    within the insertions transform collection.
+    """
+
+    def __init__(self, subtotal_dict, valid_elements, fallback_insertion_id):
         self._subtotal_dict = subtotal_dict
         self._valid_elements = valid_elements
-        self._prune = prune
+        self._fallback_insertion_id = fallback_insertion_id
 
     @lazyproperty
     def anchor(self):
@@ -963,9 +970,14 @@ class _Subtotal(object):
                 for idx, vector in enumerate(self._valid_elements)
                 if vector.element_id in addend_ids
             ),
-            dtype=int,
+            dtype=int,  # force int so it can be used as index even if empty
         )
 
+    @lazyproperty
+    def insertion_id(self):
+        """int unique identifier of this subtotal within this dimension's insertions."""
+        return self._subtotal_dict.get("insertion_id", self._fallback_insertion_id)
+
     @lazyproperty
     def label(self):
         """str display name for this subtotal, suitable for use as label."""
@@ -973,6 +985,30 @@ class _Subtotal(object):
         return name if name else ""
 
     @lazyproperty
-    def prune(self):
-        """True if this subtotal should not appear when empty."""
-        return self._prune
+    def subtrahend_ids(self):
+        """tuple of int ids of elements of negative part of the subtotal.
+
+        Any element id not present in the dimension or present but
+        representing missing data is excluded.
+        """
+        return tuple(
+            arg
+            for arg in self._subtotal_dict.get("kwargs", {}).get("negative", [])
+            if arg in self._valid_elements.element_ids
+        )
+
+    @lazyproperty
+    def subtrahend_idxs(self):
+        """ndarray of int base-element offsets contributing to the negative part of subtotal.
+
+        Suitable for directly indexing a numpy array object (such as base values or
+        margin) to extract the addend values for this subtotal.
+        """
+        return np.fromiter(
+            (
+                idx
+                for idx, vector in enumerate(self._valid_elements)
+                if vector.element_id in self.subtrahend_ids
+            ),
+            dtype=int,  # force int so it can be used as index even if empty
+        )
