@@ -2,12 +2,10 @@
 
 """Integration test suite for the cr.cube.dimension module."""
 
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 import numpy as np
 import pytest
 
-from cr.cube.crunch_cube import CrunchCube
+from cr.cube.cube import Cube
 from cr.cube.dimension import (
     AllDimensions,
     _AllElements,
@@ -17,10 +15,10 @@ from cr.cube.dimension import (
     _Subtotal,
     _ValidElements,
 )
-from cr.cube.enum import DIMENSION_TYPE as DT
+from cr.cube.enums import DIMENSION_TYPE as DT
 
 from ..fixtures import CR  # ---mnemonic: CR = 'cube-response'---
-from ..unitutil import instance_mock
+from ..unitutil import instance_mock, property_mock
 
 
 class DescribeIntegratedAllDimensions(object):
@@ -31,6 +29,10 @@ class DescribeIntegratedAllDimensions(object):
         (
             (CR.CAT_X_CAT, (DT.CAT, DT.CAT)),
             (CR.CA_X_MR_WEIGHTED_HS, (DT.CA, DT.CA_CAT, DT.MR, DT.MR_CAT)),
+            (
+                CR.MR_X_MR_SELECTED_CATEGORIES,
+                (DT.MR_SUBVAR, DT.MR_CAT, DT.MR_SUBVAR, DT.MR_CAT),
+            ),
         ),
     )
     def it_resolves_the_type_of_each_dimension(self, cube_response, expected_types):
@@ -51,6 +53,25 @@ class DescribeIntegratedAllDimensions(object):
 
         assert apparent_dimension_types == (DT.CA, DT.CA_CAT, DT.MR)
 
+    @pytest.mark.parametrize(
+        "dim_types, expected_value",
+        (
+            ((DT.NUM_ARRAY, DT.CAT), (1, 0)),
+            ((DT.CAT, DT.CA_SUBVAR), (0, 1)),
+            ((DT.NUM_ARRAY, DT.MR_SUBVAR, DT.MR_CAT), (1, 2, 0)),
+            ((DT.NUM_ARRAY,), (0,)),
+        ),
+    )
+    def it_knows_if_its_dimensions_order(self, request, dim_types, expected_value):
+        _dimensions_ = tuple(
+            instance_mock(request, Dimension, name="dim-%d" % idx, dimension_type=dt)
+            for idx, dt in enumerate(dim_types)
+        )
+        _dimensions_prop_ = property_mock(request, AllDimensions, "_dimensions")
+        _dimensions_prop_.return_value = _dimensions_
+
+        assert AllDimensions(None).dimension_order == expected_value
+
 
 class DescribeIntegratedDimension(object):
     """Integration-test suite for `cr.cube.dimension.Dimension` object."""
@@ -163,7 +184,7 @@ class DescribeIntegrated_AllElements(object):
 
     @pytest.fixture
     def type_dict(self):
-        return CrunchCube(CR.ECON_BLAME_WITH_HS).dimensions[0]._dimension_dict["type"]
+        return Cube(CR.ECON_BLAME_WITH_HS).dimensions[0]._dimension_dict["type"]
 
 
 class DescribeIntegrated_ValidElements(object):
@@ -239,7 +260,7 @@ class DescribeIntegrated_Element(object):
     @pytest.fixture
     def element_dict(self):
         return (
-            CrunchCube(CR.ECON_BLAME_WITH_HS)
+            Cube(CR.ECON_BLAME_WITH_HS)
             .dimensions[0]
             ._dimension_dict["type"]["categories"][0]
         )
