@@ -2,18 +2,18 @@
 
 """Integration-test suite for multiple-response cube behaviors"""
 
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 import numpy as np
+import pytest
 
 from cr.cube.cube import Cube
 
 from ..fixtures import CR
+from ..util import load_python_expression
 
 
 def test_labels_simple_mr_exclude_missing():
-    slice_ = Cube(CR.SIMPLE_MR).partitions[0]
-    assert slice_.row_labels == ("Response #1", "Response #2", "Response #3")
+    strand = Cube(CR.SIMPLE_MR).partitions[0]
+    assert strand.row_labels.tolist() == ["Response #1", "Response #2", "Response #3"]
 
 
 def test_as_array_simple_mr_exclude_missing():
@@ -23,8 +23,8 @@ def test_as_array_simple_mr_exclude_missing():
 
 
 def test_margin_simple_mr_axis_none():
-    slice_ = Cube(CR.SIMPLE_MR).partitions[0]
-    np.testing.assert_array_equal(slice_.table_margin, [5, 6, 6])
+    strand = Cube(CR.SIMPLE_MR).partitions[0]
+    assert strand.table_margin_range.tolist() == [5, 6]
 
 
 def test_proportions_simple_mr():
@@ -35,32 +35,26 @@ def test_proportions_simple_mr():
 
 def test_std_dev_err_simple_mr():
     strand = Cube(CR.SIMPLE_MR).partitions[0]
-
-    np.testing.assert_almost_equal(
-        strand.standard_deviation, [0.4898979, 0.4714045, 0.0]
-    )
-    np.testing.assert_almost_equal(strand.standard_error, [0.219089, 0.1924501, 0.0])
+    assert strand.table_proportion_stddevs == pytest.approx([0.4898979, 0.4714045, 0.0])
+    assert strand.table_proportion_stderrs == pytest.approx([0.219089, 0.1924501, 0.0])
 
 
 def test_1D_mr_with_means():
     strand = Cube(CR.MR_MEAN_FILT_WGTD).partitions[0]
-    np.testing.assert_almost_equal(
-        strand.means, [3.7240515, 2.5784293, 2.2185933, 1.8653349]
-    )
-    np.testing.assert_array_equal(strand.table_base, [336, 136, 248, 5216])
-    assert strand.table_base_unpruned == 23348
+
     assert strand.ndim == 1
+    assert strand.means == pytest.approx([3.7240515, 2.5784293, 2.2185933, 1.8653349])
+    assert strand.unweighted_counts.tolist() == [336, 136, 248, 5216]
+    assert strand.unweighted_bases.tolist() == [5837, 5837, 5837, 5837]
+    assert strand.table_base_range.tolist() == [5837, 5837]
 
 
 def test_deck_with_means():
-    slice_ = Cube(CR.DECK_WITH_MEAN).partitions[0]
-    np.testing.assert_array_equal(
-        slice_.table_base_unpruned, [588, 588, 588, 588, 588, 588, 588, 588]
-    )
-    np.testing.assert_almost_equal(
-        slice_.table_margin_unpruned,
-        [585.086, 585.086, 585.086, 585.086, 585.086, 585.086, 585.086, 585.086],
-    )
+    strand = Cube(CR.DECK_WITH_MEAN).partitions[0]
+
+    assert strand.ndim == 1
+    assert strand.table_base_range.tolist() == [588, 588]
+    assert strand.table_margin_range == pytest.approx([585.086, 585.086])
 
 
 def test_3D_with_means():
@@ -74,8 +68,8 @@ def test_3D_with_means():
         ],
     )
     assert slice_.table_base == 24
-    np.testing.assert_array_equal(slice_.column_base, [9, 0, 3, 3, 9])
-    np.testing.assert_array_equal(slice_.row_base, [9, 6, 9])
+    np.testing.assert_array_equal(slice_.columns_base, [9, 0, 3, 3, 9])
+    np.testing.assert_array_equal(slice_.rows_base, [9, 6, 9])
 
 
 def test_mr_x_cat_with_means():
@@ -115,8 +109,8 @@ def test_proportions_simple_mr_prune():
 
 def test_labels_cat_x_mr_exclude_missing():
     slice_ = Cube(CR.CAT_X_MR).partitions[0]
-    assert slice_.row_labels == ("rambutan", "satsuma")
-    assert slice_.column_labels == ("dog", "cat", "wombat")
+    assert tuple(slice_.row_labels) == ("rambutan", "satsuma")
+    assert tuple(slice_.column_labels) == ("dog", "cat", "wombat")
 
 
 def test_as_array_cat_x_mr():
@@ -261,125 +255,32 @@ def test_table_base_unpruned_cat_x_mr():
     np.testing.assert_array_equal(slice_.table_base_unpruned, expected)
 
 
-def test_various_measures_from_r_rows_margin():
+def test_mr_x_cat_various_measures():
     slice_ = Cube(CR.MR_X_CAT_PROFILES_STATS_WEIGHTED).partitions[0]
-    expected_zscores = [
-        [
-            -1.465585354569577,
-            3.704125875262655,
-            3.823689449491973,
-            1.53747452587281,
-            2.584734165643072,
-            -7.488143461076757,
-            -0.248968750486873,
-            0.794143540856786,
-        ],
-        [
-            1.465585354569564,
-            -3.704125875262655,
-            -3.823689449491981,
-            -1.537474525872799,
-            -2.584734165643066,
-            7.488143461076757,
-            0.248968750486873,
-            -0.794143540856781,
-        ],
-    ]
-    expected_table_std_dev = [
-        [
-            0.32529036,
-            0.3230502,
-            0.24634286,
-            0.17015146,
-            0.15631497,
-            0.310228,
-            0.07333405,
-            0.09462074,
-        ],
-        [
-            0.34030642,
-            0.32321237,
-            0.24212202,
-            0.17077218,
-            0.15232289,
-            0.34229973,
-            0.07720184,
-            0.09510867,
-        ],
-    ]
-    expected_table_std_err = [
-        [
-            0.00185406,
-            0.0018413,
-            0.00140409,
-            0.00096982,
-            0.00089095,
-            0.00176821,
-            0.00041798,
-            0.00053931,
-        ],
-        [
-            0.00193965,
-            0.00184222,
-            0.00138003,
-            0.00097335,
-            0.0008682,
-            0.00195101,
-            0.00044003,
-            0.00054209,
-        ],
-    ]
-    expected_col_std_err = [
-        [
-            0.00564723,
-            0.00585537,
-            0.00798203,
-            0.01164323,
-            0.01289225,
-            0.00573883,
-            0.02665202,
-            0.02114537,
-        ],
-        [
-            0.00564723,
-            0.00585537,
-            0.00798203,
-            0.01164323,
-            0.01289225,
-            0.00573883,
-            0.02665202,
-            0.02114537,
-        ],
-    ]
-
-    expected_col_std_dev = [
-        [
-            0.49930382,
-            0.49999992,
-            0.49991404,
-            0.49999647,
-            0.49982403,
-            0.4967613,
-            0.49933267,
-            0.49999326,
-        ],
-        [
-            0.49930382,
-            0.49999992,
-            0.49991404,
-            0.49999647,
-            0.49982403,
-            0.4967613,
-            0.49933267,
-            0.49999326,
-        ],
-    ]
-
-    np.testing.assert_almost_equal(slice_.zscores, expected_zscores)
-    np.testing.assert_almost_equal(slice_.table_std_err, expected_table_std_err)
-    np.testing.assert_almost_equal(slice_.table_std_dev, expected_table_std_dev)
-    np.testing.assert_almost_equal(slice_.columns_std_dev, expected_col_std_dev)
-    np.testing.assert_almost_equal(slice_.columns_std_err, expected_col_std_err)
+    np.testing.assert_almost_equal(
+        slice_.zscores, load_python_expression("mr-x-cat-weighted-zscores")
+    )
+    np.testing.assert_almost_equal(
+        slice_.table_std_err, load_python_expression("mr-x-cat-weighted-table-std-err")
+    )
+    np.testing.assert_almost_equal(
+        slice_.table_std_dev,
+        load_python_expression("mr-x-cat-weighted-table-std-dev"),
+    )
+    np.testing.assert_almost_equal(
+        slice_.column_std_dev, load_python_expression("mr-x-cat-weighted-col-std-dev")
+    )
+    np.testing.assert_almost_equal(
+        slice_.column_std_err, load_python_expression("mr-x-cat-weighted-col-std-err")
+    )
+    np.testing.assert_almost_equal(
+        slice_.column_proportions_moe,
+        load_python_expression("mr-x-cat-weighted-col-moe"),
+    )
+    np.testing.assert_almost_equal(
+        slice_.table_proportions_moe,
+        load_python_expression("mr-x-cat-weighted-table-moe"),
+    )
 
 
 def test_mr_x_single_wave():
@@ -442,13 +343,21 @@ def test_std_deviation_std_error_array_x_mr_by_row():
         [[0.02978762, 0.00971635, 0.03292998], [0.02918338, 0.03472281, 0.02929588]],
     )
     np.testing.assert_array_almost_equal(
-        slice_.columns_std_dev,
+        slice_.table_proportions_moe,
+        [[0.05838266, 0.01904371, 0.06454158], [0.05719838, 0.06805546, 0.05741888]],
+    )
+    np.testing.assert_array_almost_equal(
+        slice_.column_std_dev,
         [[0.49978635, 0.20331906, 0.49121125], [0.49978635, 0.20331906, 0.49121125]],
     )
     np.testing.assert_array_almost_equal(
-        slice_.columns_std_err,
+        slice_.column_std_err,
         [[0.05158518, 0.02113084, 0.04615627], [0.05158518, 0.02113084, 0.04615627]],
     )
+    np.testing.assert_array_almost_equal(
+        slice_.column_proportions_moe,
+        [[0.1011051, 0.04141569, 0.09046463], [0.1011051, 0.04141569, 0.09046463]],
+    )
 
 
 def test_array_x_mr_by_cell():
@@ -462,77 +371,6 @@ def test_simple_mr_margin_by_col():
     np.testing.assert_array_equal(slice_.rows_margin, [3, 4, 0])
 
 
-def test_cat_x_mr_aug_zscores():
-    slice_ = Cube(CR.EDU_FAV5_FAV5).partitions[0]
-
-    assert slice_.cube_is_mr_aug is True
-    np.testing.assert_array_almost_equal(
-        slice_.zscores,
-        [
-            [-2.965933, -6.078075, -8.307031, -6.154084, -5.897443],
-            [-0.739747, -2.274391, -0.147797, -2.444906, -1.607212],
-            [2.81708, 5.763211, 6.28017, 5.28286, 5.739841],
-            [2.205063, 5.632647, 5.292887, 6.565961, 4.418183],
-        ],
-    )
-    np.testing.assert_array_almost_equal(
-        slice_.table_std_dev,
-        [
-            [0.21372868, 0.22934489, 0.25231982, 0.2196406, 0.29240244],
-            [0.21570152, 0.24252658, 0.29683733, 0.23332739, 0.30050586],
-            [0.19356893, 0.24632529, 0.2755421, 0.23750454, 0.28209093],
-            [0.15827103, 0.21101155, 0.22996442, 0.21354113, 0.23220219],
-        ],
-    )
-    np.testing.assert_array_almost_equal(
-        slice_.table_std_err,
-        [
-            [0.00552364, 0.00592723, 0.006521, 0.00567643, 0.0075569],
-            [0.00557463, 0.0062679, 0.00767152, 0.00603016, 0.00776633],
-            [0.00500263, 0.00636608, 0.00712116, 0.00613811, 0.00729041],
-            [0.00409039, 0.00545342, 0.00594324, 0.0055188, 0.00600108],
-        ],
-    )
-    np.testing.assert_almost_equal(
-        slice_.columns_std_dev,
-        [
-            [0.45689893, 0.42836987, 0.41706167, 0.4238419, 0.44819968],
-            [0.45943524, 0.44539695, 0.46662527, 0.44232528, 0.45647742],
-            [0.42785093, 0.44999726, 0.44477969, 0.44760084, 0.43699848],
-            [0.36576824, 0.40225385, 0.38733066, 0.41506083, 0.37441564],
-        ],
-    )
-    np.testing.assert_almost_equal(
-        slice_.columns_std_err,
-        [
-            [0.02937373, 0.02308169, 0.01952406, 0.02354123, 0.01988999],
-            [0.02953678, 0.02399915, 0.0218443, 0.02456784, 0.02025734],
-            [0.02750625, 0.02424703, 0.02082163, 0.02486086, 0.01939291],
-            [0.023515, 0.02167449, 0.01813225, 0.02305351, 0.01661564],
-        ],
-    )
-
-
-def test_cat_x_mr_and_cat_x_mr_augmented_various_measures():
-    slice_ = Cube(CR.EDU_FAV5).partitions[0]
-    slice2_ = Cube(CR.EDU_FAV5_FAV5).partitions[0]
-
-    np.testing.assert_array_almost_equal(
-        slice_.column_proportions, slice2_.column_proportions
-    )
-    np.testing.assert_array_almost_equal(slice_.zscores, slice2_.zscores)
-    np.testing.assert_array_almost_equal(slice_.table_std_dev, slice2_.table_std_dev)
-    np.testing.assert_array_almost_equal(slice_.table_std_err, slice2_.table_std_err)
-    np.testing.assert_array_almost_equal(
-        slice_.columns_std_dev, slice2_.columns_std_dev
-    )
-    np.testing.assert_array_almost_equal(
-        slice_.columns_std_err, slice2_.columns_std_err
-    )
-    assert slice_.shape == (4, 5)
-    assert slice2_.shape == (4, 5)
-
-
 def test_cat_x_mr_x_mr_proportions_by_row():
     slice_ = Cube(CR.CAT_X_MR_X_MR).partitions[0]
     np.testing.assert_almost_equal(
@@ -552,6 +390,42 @@ def test_cat_x_mr_x_mr_proportions_by_row():
     )
 
 
+def test_cat_x_mr_x_mr_row_std_err():
+    slice_ = Cube(CR.CAT_X_MR_X_MR).partitions[0]
+
+    row_std_err = slice_.row_std_err
+
+    assert row_std_err.tolist() == [
+        [0.005062449141597438, 0.006313386789860505],
+        [0.012489721142934502, 0.015435416534978305],
+        [0.012747222182198531, 0.015713252542485612],
+    ]
+
+
+def test_cat_x_mr_x_mr_column_std_err():
+    slice_ = Cube(CR.CAT_X_MR_X_MR).partitions[0]
+
+    column_std_err = slice_.column_std_err
+
+    assert column_std_err.tolist() == [
+        [0.011171263445095775, 0.006336762995064969],
+        [0.006945529504027231, 0.0039102714263034456],
+        [0.006866798249349935, 0.0038462753714309968],
+    ]
+
+
+def test_cat_x_mr_x_mr_table_std_err():
+    slice_ = Cube(CR.CAT_X_MR_X_MR).partitions[0]
+
+    table_std_err = slice_.table_std_err
+
+    assert table_std_err.tolist() == [
+        [0.003201049671592117, 0.004799123982561817],
+        [0.0013896729831150924, 0.0023822644689454613],
+        [0.0013722740251130602, 0.0023412125063735673],
+    ]
+
+
 def test_cat_x_mr_x_mr_pruned_rows():
     # Not pruned
     slice_ = Cube(CR.CAT_X_MR_X_MR_PRUNED_ROWS).partitions[0]
@@ -803,7 +677,7 @@ def test_mr_x_cat_x_mr_pruned_rows():
 
 def test_mr_x_num_with_means_pruned():
     transforms = {"columns_dimension": {"prune": True}}
-    slice_ = Cube(CR.BBC_NEWS, transforms=transforms).partitions[0]
+    slice_ = Cube(CR.MEANS_MR_X_CAT, transforms=transforms).partitions[0]
     expected = [
         [
             38.79868092168848,
@@ -899,21 +773,8 @@ def test_mr_x_num_with_means_pruned():
     np.testing.assert_almost_equal(slice_.means, expected)
 
 
-def test_mr_x_mr_augmented_zscores():
-    slice_ = Cube(CR.MR_X_MR_AUGMENTED).partitions[0]
-    expected = [
-        [2.02358771, 4.40571894, -4.73781476],
-        [1.6374635, 2.98524415, -2.88925509],
-        [1.40611428, -0.26969232, -1.03556062],
-        [1.67279974, -0.46401375, 0.06965056],
-        [-3.37595335, -1.34855138, 3.10157807],
-    ]
-
-    np.testing.assert_almost_equal(slice_.zscores, expected)
-
-
 def test_mr_x_num_with_means_not_pruned():
-    slice_ = Cube(CR.BBC_NEWS).partitions[0]
+    slice_ = Cube(CR.MEANS_MR_X_CAT).partitions[0]
     expected = [
         [
             38.79868092,
@@ -1046,22 +907,22 @@ def test_mr_x_num_with_means_not_pruned():
 
 
 def test_mr_x_num_rows_margin():
-    slice_ = Cube(CR.BBC_NEWS).partitions[0]
+    slice_ = Cube(CR.MEANS_MR_X_CAT).partitions[0]
     expected = [4805, 3614, 1156, 1200, 644, 258, 167, 170, 11419]
-    np.testing.assert_array_equal(slice_.row_base, expected)
+    np.testing.assert_array_equal(slice_.rows_base, expected)
 
 
 def test_mr_x_num_cols_margin_not_pruned_unweighted():
-    slice_ = Cube(CR.BBC_NEWS).partitions[0]
+    slice_ = Cube(CR.MEANS_MR_X_CAT).partitions[0]
     expected = [1728, 1523, 1570, 1434, 1459, 1429, 1461, 1432, 0, 0, 0, 0]
-    np.testing.assert_array_equal(slice_.column_base[0], expected)
+    np.testing.assert_array_equal(slice_.columns_base[0], expected)
 
 
 def test_mr_x_num_cols_margin_pruned_unweighted():
     transforms = {"columns_dimension": {"prune": True}}
-    slice_ = Cube(CR.BBC_NEWS, transforms=transforms).partitions[0]
+    slice_ = Cube(CR.MEANS_MR_X_CAT, transforms=transforms).partitions[0]
     expected = [1728, 1523, 1570, 1434, 1459, 1429, 1461, 1432]
-    np.testing.assert_array_equal(slice_.column_base[0], expected)
+    np.testing.assert_array_equal(slice_.columns_base[0], expected)
 
 
 def test_num_x_mr_props_by_row():
@@ -1545,7 +1406,7 @@ def test_cat_by_mr_hs_cell_percentage():
 
 
 def test_mr_by_cat_hs_col_percentage():
-    slice_ = Cube(CR.MR_X_CAT_HS).partitions[0]
+    slice_ = Cube(CR.MR_X_CAT_HS_MT).partitions[0]
     expected = [
         [
             0.6399160598631669,
@@ -1602,7 +1463,7 @@ def test_mr_by_cat_hs_col_percentage():
 
 
 def test_mr_by_cat_hs_row_percentage():
-    slice_ = Cube(CR.MR_X_CAT_HS).partitions[0]
+    slice_ = Cube(CR.MR_X_CAT_HS_MT).partitions[0]
     expected = [
         [
             0.44079255048452126,
@@ -1659,7 +1520,7 @@ def test_mr_by_cat_hs_row_percentage():
 
 
 def test_mr_by_cat_hs_cell_percentage():
-    slice_ = Cube(CR.MR_X_CAT_HS).partitions[0]
+    slice_ = Cube(CR.MR_X_CAT_HS_MT).partitions[0]
     expected = [
         [
             0.07905704201278009,
@@ -1716,7 +1577,6 @@ def test_mr_by_cat_hs_cell_percentage():
 
 
 def test_mr_x_cat_min_base_size_mask():
-
     transforms = {
         "columns_dimension": {"insertions": {}},
         "rows_dimension": {"insertions": {}},
@@ -1729,7 +1589,7 @@ def test_mr_x_cat_min_base_size_mask():
     # We thus choose the min base size to be 220, and expeect it to broadcast across
     # columns (in the row direction, i.e. axis=1), sincee the MR is what won't be
     # collapsed after doing the base calculation in the table direction.
-    slice_ = Cube(CR.MR_X_CAT_HS, transforms=transforms, mask_size=220).partitions[0]
+    slice_ = Cube(CR.MR_X_CAT_HS_MT, transforms=transforms, mask_size=220).partitions[0]
 
     expected = np.array(
         [
@@ -1755,7 +1615,7 @@ def test_mr_x_cat_min_base_size_mask():
     # )
     #
     # We thus choose the min base size to be 30, and expeect it to not be broadcast.
-    slice_ = Cube(CR.MR_X_CAT_HS, transforms=transforms, mask_size=30).partitions[0]
+    slice_ = Cube(CR.MR_X_CAT_HS_MT, transforms=transforms, mask_size=30).partitions[0]
 
     expected_column_mask = np.array(
         [
@@ -1777,7 +1637,7 @@ def test_mr_x_cat_min_base_size_mask():
     # We thus choose the min base size to be 80, and expeect it to broadcast across
     # columns (in the row direction, i.e. axis=1), sincee the MR is what won't be
     # collapsed after doing the base calculation in the row direction.
-    slice_ = Cube(CR.MR_X_CAT_HS, transforms=transforms, mask_size=80).partitions[0]
+    slice_ = Cube(CR.MR_X_CAT_HS_MT, transforms=transforms, mask_size=80).partitions[0]
 
     expected_row_mask = np.array(
         [
