@@ -2,7 +2,7 @@
 
 """Unit test suite for `cr.cube.cubepart` module."""
 
-from __future__ import absolute_import, division, print_function, unicode_literals
+from __future__ import division
 
 import numpy as np
 import pytest
@@ -10,9 +10,9 @@ import pytest
 from cr.cube.cube import Cube
 from cr.cube.cubepart import CubePartition, _Slice, _Strand, _Nub
 from cr.cube.dimension import Dimension
-from cr.cube.enum import DIMENSION_TYPE as DT
-from cr.cube.matrix import TransformedMatrix, _VectorAfterHiding
-from cr.cube.stripe import _BaseStripeRow, TransformedStripe
+from cr.cube.matrix import Assembler
+from cr.cube.noa.smoothing import SingleSidedMovingAvgSmoother
+from cr.cube.stripe.assembler import StripeAssembler
 
 from ..unitutil import class_mock, instance_mock, property_mock
 
@@ -64,29 +64,36 @@ class DescribeCubePartition(object):
 
         assert cube_index == 42
 
-    def it_knows_if_cube_is_mr_aug(self):
-        # --- default of False is overridden by subclasses when appropriate ---
-        assert CubePartition(None).cube_is_mr_aug is False
-
-    @pytest.mark.parametrize(
-        "dims, expected_value",
-        (
-            ((DT.CAT, DT.MR, DT.MR_CAT), DT.CAT),
-            ((DT.MR, DT.MR_CAT), DT.MR),
-            ((DT.MR_CAT,), DT.MR_CAT),
-        ),
-    )
-    def it_knows_cube_row_dimension_type(self, request, cube_, dims, expected_value):
-        all_dimensions_ = tuple(
-            instance_mock(request, Dimension, name="dim-%d" % idx, dimension_type=dt)
-            for idx, dt in enumerate(dims)
+    def it_can_evaluate_a_measure_expression(self, request):
+        single_sided_moving_avg_smoother_ = instance_mock(
+            request, SingleSidedMovingAvgSmoother, values=[[0.1, 0.2], [0.3, 0.4]]
         )
-        cube_.dimensions = all_dimensions_
-        cube_partition = CubePartition(cube_)
+        SingleSidedMovingAvgSmoother_ = class_mock(
+            request,
+            "cr.cube.cubepart.SingleSidedMovingAvgSmoother",
+            return_value=single_sided_moving_avg_smoother_,
+        )
+        measure_expr = {
+            "function": "one_sided_moving_avg",
+            "base_measure": "col_percent",
+            "window": 2,
+        }
+        cube_partition = CubePartition(None, None)
 
-        cube_row_dimension_type = cube_partition.cube_row_dimension_type
+        values = cube_partition.evaluate(measure_expr)
 
-        assert cube_row_dimension_type == expected_value
+        SingleSidedMovingAvgSmoother_.assert_called_once_with(
+            cube_partition, measure_expr
+        )
+        assert values == [[0.1, 0.2], [0.3, 0.4]]
+
+    def but_it_raises_an_exception_when_function_is_not_available(self):
+        slice_ = _Slice(None, None, None, None, None)
+
+        with pytest.raises(NotImplementedError) as err:
+            slice_.evaluate({"function": "F"})
+
+        assert str(err.value) == "Function F is not available."
 
     def it_knows_the_primary_alpha_value_to_help(self, _alpha_values_prop_):
         """alpha is the primary confidence-interval threshold specified by the user."""
@@ -219,6 +226,20 @@ class DescribeCubePartition(object):
 class Describe_Slice(object):
     """Unit test suite for `cr.cube.cubepart._Slice` object."""
 
+    def it_knows_the_column_unweighted_bases(self, _assembler_prop_, assembler_):
+        _assembler_prop_.return_value = assembler_
+        assembler_.column_unweighted_bases = np.array([[0, 1], [2, 3]])
+        slice_ = _Slice(None, None, None, None, None)
+
+        assert slice_.column_unweighted_bases.tolist() == [[0, 1], [2, 3]]
+
+    def it_knows_the_column_weighted_bases(self, _assembler_prop_, assembler_):
+        _assembler_prop_.return_value = assembler_
+        assembler_.column_weighted_bases = np.array([[0.0, 1.1], [2.2, 3.3]])
+        slice_ = _Slice(None, None, None, None, None)
+
+        assert slice_.column_weighted_bases.tolist() == [[0.0, 1.1], [2.2, 3.3]]
+
     @pytest.mark.parametrize(
         ("shape", "expected_value"),
         (((4, 2), False), ((4, 0), True), ((0, 2), True), ((0, 0), True)),
@@ -258,31 +279,59 @@ class Describe_Slice(object):
 
         assert population_fraction == 0.5
 
-    def it_knows_the_row_proportions(self, request, _matrix_prop_, matrix_):
-        _matrix_prop_.return_value = matrix_
-        matrix_.rows = (
-            instance_mock(request, _VectorAfterHiding, proportions=(0.1, 0.2, 0.3)),
-            instance_mock(request, _VectorAfterHiding, proportions=(0.4, 0.5, 0.6)),
+    def it_knows_the_row_unweighted_bases(self, _assembler_prop_, assembler_):
+        _assembler_prop_.return_value = assembler_
+        assembler_.row_unweighted_bases = np.array([[3, 2], [1, 0]])
+        slice_ = _Slice(None, None, None, None, None)
+
+        assert slice_.row_unweighted_bases.tolist() == [[3, 2], [1, 0]]
+
+    def it_knows_the_row_weighted_bases(self, _assembler_prop_, assembler_):
+        _assembler_prop_.return_value = assembler_
+        assembler_.row_weighted_bases = np.array([[3.3, 2.2], [1.1, 0.0]])
+        slice_ = _Slice(None, None, None, None, None)
+
+        assert slice_.row_weighted_bases.tolist() == [[3.3, 2.2], [1.1, 0.0]]
+
+    def it_knows_the_rows_margin(self, _assembler_prop_, assembler_):
+        _assembler_prop_.return_value = assembler_
+        assembler_.rows_margin = [[1, 2], [3, 4]]
+
+        assert _Slice(None, None, None, None, None).rows_margin == [[1, 2], [3, 4]]
+
+    def it_knows_the_row_proportion(self, _assembler_prop_, assembler_):
+        _assembler_prop_.return_value = assembler_
+        assembler_.row_proportions = [[1, 2], [3, 4]]
+
+        assert _Slice(None, None, None, None, None).row_proportions == [[1, 2], [3, 4]]
+
+    def it_knows_the_scale_means_column(
+        self, request, _columns_dimension_numeric_values_prop_
+    ):
+        _columns_dimension_numeric_values_prop_.return_value = np.array([1, np.nan, 3])
+        property_mock(
+            request,
+            _Slice,
+            "row_proportions",
+            return_value=(
+                (np.array([[20, 30, 12], [10, 8, 4]]).T / np.array([62, 22])).T
+            ),
         )
         slice_ = _Slice(None, None, None, None, None)
 
-        row_proportions = slice_.row_proportions
+        rows_scale_means = slice_.rows_scale_mean
 
-        np.testing.assert_almost_equal(
-            row_proportions, [[0.1, 0.2, 0.3], [0.4, 0.5, 0.6]]
-        )
+        np.testing.assert_almost_equal(rows_scale_means, [1.75, 1.57142857])
 
-    def it_knows_the_rows_margin(self, request, _matrix_prop_, matrix_):
-        _matrix_prop_.return_value = matrix_
-        matrix_.rows = (
-            instance_mock(request, _VectorAfterHiding, margin=(1, 2)),
-            instance_mock(request, _VectorAfterHiding, margin=(3, 4)),
+    def but_the_scale_means_column_is_None_if_columns_have_no_numeric_values(
+        self, _columns_dimension_numeric_values_prop_
+    ):
+        _columns_dimension_numeric_values_prop_.return_value = np.array(
+            [np.nan, np.nan, np.nan]
         )
         slice_ = _Slice(None, None, None, None, None)
 
-        rows_margin = slice_.rows_margin
-
-        np.testing.assert_almost_equal(rows_margin, [[1, 2], [3, 4]])
+        assert slice_.rows_scale_mean is None
 
     def it_provides_the_secondary_scale_mean_pairwise_indices(
         self, _alpha_alt_prop_, _only_larger_prop_, PairwiseSignificance_
@@ -296,39 +345,87 @@ class Describe_Slice(object):
         _only_larger_prop_.return_value = True
         slice_ = _Slice(None, None, None, None, None)
 
-        scale_mean_pairwise_indices_alt = slice_.scale_mean_pairwise_indices_alt
+        columns_scale_mean_pw_idxs_alt = slice_.columns_scale_mean_pairwise_indices_alt
 
         PairwiseSignificance_.scale_mean_pairwise_indices.assert_called_once_with(
             slice_, 0.42, True
         )
-        assert scale_mean_pairwise_indices_alt == ((2,), (0,), ())
+        assert columns_scale_mean_pw_idxs_alt == ((2,), (0,), ())
 
-    def but_scale_mean_pairwise_indices_alt_is_None_when_no_secondary_alpha_specified(
+    def but_columns_scale_mean_pairwise_indices_alt_is_None_when_no_secondary_alpha_specified(
         self, _alpha_alt_prop_
     ):
         _alpha_alt_prop_.return_value = None
         slice_ = _Slice(None, None, None, None, None)
 
-        assert slice_.scale_mean_pairwise_indices_alt is None
+        assert slice_.columns_scale_mean_pairwise_indices_alt is None
 
-    def it_can_evaluate_a_measure_expression(
-        self, col_percent_prop, smoothed_dimension_dict_prop_
+    @pytest.mark.parametrize(
+        "dimensions_dicts, expected_value",
+        (
+            ([{"references": {}}], ()),
+            (
+                [
+                    {"references": {}},
+                    {
+                        "references": {
+                            "selected_categories": [{"name": "Foo"}, {"name": "Bar"}]
+                        }
+                    },
+                ],
+                ("Foo", "Bar"),
+            ),
+            (
+                [
+                    {"references": {}},
+                    {
+                        "references": {
+                            "selected_categories": [{"name": "Foo"}, {"id": "Bar"}]
+                        }
+                    },
+                ],
+                ("Foo",),
+            ),
+        ),
+    )
+    def it_knows_its_selected_category_labels(
+        self, _dimensions_prop_, dimensions_dicts, expected_value
     ):
-        slice_ = _Slice(None, None, None, None, None)
-        col_percent_prop.return_value = np.array([[0.1, 0.2], [0.3, 0.4]])
-        smoothed_dimension_dict_prop_.return_value = {"type": {"categories": []}}
-
-        evaluate = slice_.evaluate({"function": "one_sided_moving_avg"})
-
-        np.testing.assert_almost_equal(evaluate, [[0.1, 0.2], [0.3, 0.4]])
-
-    def but_it_raises_an_exception_when_function_is_not_available(self):
+        _dimensions_prop_.return_value = [
+            Dimension(dim_dict, None, None) for dim_dict in dimensions_dicts
+        ]
         slice_ = _Slice(None, None, None, None, None)
 
-        with pytest.raises(NotImplementedError) as err:
-            slice_.evaluate({"function": "F"})
+        assert slice_.selected_category_labels == expected_value
 
-        assert str(err.value) == "Function F is not available."
+    def it_knows_the_table_unweighted_bases(self, _assembler_prop_, assembler_):
+        _assembler_prop_.return_value = assembler_
+        assembler_.table_unweighted_bases = np.array([[2, 3], [0, 1]])
+        slice_ = _Slice(None, None, None, None, None)
+
+        assert slice_.table_unweighted_bases.tolist() == [[2, 3], [0, 1]]
+
+    def it_knows_the_table_weighted_bases(self, _assembler_prop_, assembler_):
+        _assembler_prop_.return_value = assembler_
+        assembler_.table_weighted_bases = np.array([[2.2, 3.3], [0.0, 1.1]])
+        slice_ = _Slice(None, None, None, None, None)
+
+        assert slice_.table_weighted_bases.tolist() == [[2.2, 3.3], [0.0, 1.1]]
+
+    def it_constructs_its_assembler_instance_to_help(
+        self, request, cube_, _dimensions_prop_, dimension_, assembler_
+    ):
+        Assembler_ = class_mock(
+            request, "cr.cube.cubepart.Assembler", return_value=assembler_
+        )
+        _dimensions_prop_.return_value = (dimension_, dimension_)
+        slice_idx = 42
+        slice_ = _Slice(cube_, slice_idx, None, None, None)
+
+        assembler = slice_._assembler
+
+        Assembler_.assert_called_once_with(cube_, (dimension_, dimension_), slice_idx)
+        assert assembler is assembler_
 
     # fixture components ---------------------------------------------
 
@@ -336,6 +433,18 @@ class Describe_Slice(object):
     def _alpha_alt_prop_(self, request):
         return property_mock(request, _Slice, "_alpha_alt")
 
+    @pytest.fixture
+    def assembler_(self, request):
+        return instance_mock(request, Assembler)
+
+    @pytest.fixture
+    def _assembler_prop_(self, request):
+        return property_mock(request, _Slice, "_assembler")
+
+    @pytest.fixture
+    def _columns_dimension_numeric_values_prop_(self, request):
+        return property_mock(request, _Slice, "_columns_dimension_numeric_values")
+
     @pytest.fixture
     def cube_(self, request):
         return instance_mock(request, Cube)
@@ -348,14 +457,6 @@ class Describe_Slice(object):
     def _dimensions_prop_(self, request):
         return property_mock(request, _Slice, "_dimensions")
 
-    @pytest.fixture
-    def matrix_(self, request):
-        return instance_mock(request, TransformedMatrix)
-
-    @pytest.fixture
-    def _matrix_prop_(self, request):
-        return property_mock(request, _Slice, "_matrix")
-
     @pytest.fixture
     def _only_larger_prop_(self, request):
         return property_mock(request, _Slice, "_only_larger")
@@ -368,29 +469,16 @@ class Describe_Slice(object):
     def shape_prop_(self, request):
         return property_mock(request, _Slice, "shape")
 
-    @pytest.fixture
-    def col_percent_prop(self, request):
-        return property_mock(request, _Slice, "column_percentages")
-
-    @pytest.fixture
-    def smoothed_dimension_dict_prop_(self, request):
-        return property_mock(request, _Slice, "smoothed_dimension_dict")
-
 
 class Describe_Strand(object):
     """Unit test suite for `cr.cube.cubepart._Strand` object."""
 
-    def it_knows_which_of_its_rows_are_inserted(self, request, _stripe_prop_, stripe_):
-        stripe_.rows = tuple(
-            instance_mock(request, _BaseStripeRow, is_inserted=bool(i % 2))
-            for i in range(7)
-        )
-        _stripe_prop_.return_value = stripe_
+    def it_knows_which_of_its_rows_are_inserted(self, _assembler_prop_, assembler_):
+        _assembler_prop_.return_value = assembler_
+        assembler_.inserted_row_idxs = (1, 3, 5)
         strand = _Strand(None, None, None, None, None, None)
 
-        inserted_row_idxs = strand.inserted_row_idxs
-
-        assert inserted_row_idxs == (1, 3, 5)
+        assert strand.inserted_row_idxs == (1, 3, 5)
 
     @pytest.mark.parametrize(("shape", "expected_value"), (((1,), False), ((0,), True)))
     def it_knows_whether_it_is_empty(self, shape, expected_value, shape_prop_):
@@ -409,19 +497,6 @@ class Describe_Strand(object):
 
         assert title == "Unmarried"
 
-    def it_constructs_its_underlying_stripe_to_help(
-        self, request, cube_, _rows_dimension_prop_, dimension_, stripe_
-    ):
-        TransformedStripe_ = class_mock(request, "cr.cube.cubepart.TransformedStripe")
-        TransformedStripe_.stripe.return_value = stripe_
-        _rows_dimension_prop_.return_value = dimension_
-        strand = _Strand(cube_, None, None, False, 42, None)
-
-        stripe = strand._stripe
-
-        TransformedStripe_.stripe.assert_called_once_with(cube_, dimension_, False, 42)
-        assert stripe is stripe_
-
     def it_knows_the_population_fraction(self, cube_):
         cube_.population_fraction = 0.5
         strand_ = _Strand(cube_, None, None, None, None, None)
@@ -430,42 +505,41 @@ class Describe_Strand(object):
 
         assert population_fraction == 0.5
 
+    def it_knows_its_selected_categories_labels(self, _dimensions_prop_):
+        _dimensions_prop_.return_value = [Dimension({"references": {}}, None, None)]
+        strand_ = _Strand(None, None, None, None, None, None)
+
+        assert strand_.selected_category_labels == ()
+
     # fixture components ---------------------------------------------
 
+    @pytest.fixture
+    def assembler_(self, request):
+        return instance_mock(request, StripeAssembler)
+
+    @pytest.fixture
+    def _assembler_prop_(self, request):
+        return property_mock(request, _Strand, "_assembler")
+
     @pytest.fixture
     def cube_(self, request):
         return instance_mock(request, Cube)
 
-    @pytest.fixture
-    def dimension_(self, request):
-        return instance_mock(request, Dimension)
-
     @pytest.fixture
     def _dimensions_prop_(self, request):
         return property_mock(request, _Strand, "_dimensions")
 
-    @pytest.fixture
-    def _rows_dimension_prop_(self, request):
-        return property_mock(request, _Strand, "_rows_dimension")
-
     @pytest.fixture
     def shape_prop_(self, request):
         return property_mock(request, _Strand, "shape")
 
-    @pytest.fixture
-    def stripe_(self, request):
-        return instance_mock(request, TransformedStripe)
-
-    @pytest.fixture
-    def _stripe_prop_(self, request):
-        return property_mock(request, _Strand, "_stripe")
-
 
 class Describe_Nub(object):
     """Unit test suite for `cr.cube.cubepart._Nub` object."""
 
     @pytest.mark.parametrize(
-        "unweighted_count, expected_value", ((None, True), (45.4, False))
+        "unweighted_count, expected_value",
+        ((float("NaN"), True), (45.4, False), (0.0, True)),
     )
     def it_knows_when_it_is_empty(self, request, unweighted_count, expected_value):
         property_mock(request, _Nub, "unweighted_count", return_value=unweighted_count)
@@ -475,5 +549,7 @@ class Describe_Nub(object):
 
         assert is_empty == expected_value
 
-    def it_knows_its_cube_is_never_mr_aug(self):
-        assert _Nub(None).cube_is_mr_aug is False
+    def it_knows_its_selected_categories_labels(self):
+        nub_ = _Nub(None)
+
+        assert nub_.selected_category_labels == ()
